@@ -2,6 +2,7 @@
 usbhub --disable-i2c --hub D9D1 power state --port 2 --reset
 sleep 1s
 dfu-util --device 1fc9:000c --alt 0 --download firmware/hackrf_usb/build/hackrf_usb.dfu
+sleep 1s
 EXIT_CODE="$?"
 if [ "$EXIT_CODE" == "0" ]
 then
@@ -16,6 +17,6 @@ then
     echo "dfu-util installation failed! Exiting.."
     exit $EXIT_CODE
 else
-    echo "god have mercy on your soul"
+    echo "Unhandled exception"
     exit $EXIT_CODE
-fi
\ No newline at end of file
+fi
